/*********************************************************************
 * 
 *    Filename:  network.c
 *    Author:    Ryan Shillington <shillington@writeme.com>
 *    Description: Functions to manipulate the network data.
 *  
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 ********************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "network.h"

void writeIfcfgFile(byte netmask[4], byte ip[4], char *device)
{
  int ifcfg;      // The /etc/sysconfig/network-scripts/ifcfg-ethx file
  char data[512];
  char *fullFileName;
  int find=0;
  byte broadcast[4];
  byte network[4];

  //* Open the file
  fullFileName = (device[3] == '0' ? IFCFG_ETH0_PATH : IFCFG_ETH1_PATH);
  if(unlink(fullFileName) || 
      ((ifcfg = open(fullFileName, O_RDWR | O_CREAT)) == 0))
  {
    fprintf(stderr, "Error: Can't open/delete %s\n", fullFileName);
    return;
  }

  //* Figure out the broadcast and network address
  for(find = 0; find <4; find++)
  {
    broadcast[find] = ip[find] | (~netmask[find]);
    network[find] = ip[find] & netmask[find];
  }

  //* Organize the data appropriately
  sprintf(data, 
      "DEVICE=%s\nONBOOT=yes\nBROADCAST=%d.%d.%d.%d\nNETWORK=%d.%d.%d.%d\nNETMASK=%d.%d.%d.%d\nIPADDR=%d.%d.%d.%d\nBOOTPROTO=none\n",
      device,
      broadcast[0], broadcast[1], broadcast[2], broadcast[3], 
      network[0], network[1], network[2], network[3], 
      netmask[0], netmask[1], netmask[2], netmask[3], 
      ip[0], ip[1], ip[2], ip[3]);

  //* Write the new data to the file
  write(ifcfg, data, strlen(data));

  //* Close the files
  close(ifcfg);
}

void writeNetworkFile(char *name, byte gateway[4], char *device)
{
  int network;      // The /etc/sysconfig/network file
  char data[512];
  char hostName[256];
  char *domainName;
  int find=0;

  //* Open the file
  if(unlink(NETWORK_PATH) 
      || ((network = open(NETWORK_PATH, O_RDWR | O_CREAT)) == 0))
  {
    fprintf(stderr, "Error: Can't open %s\n", NETWORK_PATH);
    return;
  }
  
  //* Parse the domain name and host name
  memset(hostName, 0, 256);
  strcpy(hostName, name);
  while((hostName[find] != '.') && (hostName[find] != '\0')) find++;
  hostName[find] = 0;
  domainName=hostName+find+1;
  
  //* Organize the data appropriately
  sprintf(data, 
      "NETWORKING=yes\nHOSTNAME=%s\nDOMAINNAME=%s\nGATEWAY=%d.%d.%d.%d\nGATEWAYDEV=%s\nFORWARD_IPV4=yes\n",
      hostName,
      domainName,
      gateway[0], gateway[1], gateway[2], gateway[3], 
      device);
  
  //* Write the new data to the file
  write(network, data, strlen(data));
  
  //* Close the files
  close(network);
}

void writeHostsFile(byte ip[4], char *name, char* alias)
{
  int hosts;      // The /etc/hosts file
  char data[4096];
  char temp[4096];
  char *findp;
  int find, start, size;

  //* Open the /etc/hosts file
  if((hosts = open(HOSTS_PATH, O_RDWR | O_CREAT)) == 0)
  {
    fprintf(stderr, "Error: Can't open %s\n", HOSTS_PATH);
  }
  
  //* Read the hosts file
  size = read(hosts, data, 4096);
  data[size+1] = 0;

  //* Locate the name that we're looking for, and deal with it if it's not
  //there.
  start = locateName(data, size, name);
  if(start == -1)
  {
    //* The name isn't in the hosts file, so maybe the IP address is in here
    sprintf(temp, "%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
    if((findp = strstr(data, temp)) == NULL)
    {
      //* Yikes... no entry in the hosts file whatsoever.  So we'll make a new
      //entry.
      size--;
      while (data[size] == '\n') size--;
      size +=2;
      sprintf(data + size, "%d.%d.%d.%d %s %s\n", 
          ip[0], ip[1], ip[2], ip[3],
          name, alias);

      //* Write the new data to the file
      lseek(hosts, 0, SEEK_SET);
      write(hosts, data, strlen(data));
  
      //* Close the files
      close(hosts);

      return;
    }
    start = findp - data;
  }
  
  //* Backup the rest of the data
  find = start;
  while(data[find] != '\n') find++;
  find++;
  strncpy(temp, data+find, size-find);
  
  printf("Start = %d, find = %d, size = %d, data = %s|temp = %s", start, find, size, data,
      temp);

  //* Put the new data in
  sprintf(data + start, 
      "%d.%d.%d.%d %s %s\n", 
        ip[0], ip[1], ip[2], ip[3], 
        name, 
        alias);
  
  //* Put the rest of the data back into the buffer
  start = strlen(data);
  strncat(data, temp, size-find);

  //* Write the new data to the file
  lseek(hosts, 0, SEEK_SET);
  write(hosts, data, strlen(data));
  
  //* Close the files
  close(hosts);
}
  
//* A helper function that instantly finds the name that we're looking inside
//* the given data.  
//* 
//* We return the number of characters to the beginning of the line that the
//* data is on. 
//*
//* This function returns -1 if the string was not found.

int locateName(char data[4096], int size, char *name)
{
  int find = 0;
  char *found;
  
  //* 1 - Find the substring
  found = strstr(data, name);
  if(found == NULL) return -1;

  find = (int)(found-data);
  while(data[find] != '\n') find--;

  return (find+1);
}
